//Logger (to stream or file) v.3.1 


#ifndef _proglog_h_
#define _proglog_h_


#include <fstream> 
#include <string>
#include <time.h>
#include <stdio.h>


// auxiliary class for writing block of data to stream
class _ProgLog
{
public:
        template <class T>  
        _ProgLog& operator << (const T& src)
        { 
                ProgLog::Write(*ProgLogStream, src, false);

                return *this;
        }

private:        
        _ProgLog(std::ostream *stream) : ProgLogStream(stream) {}

        std::ostream *ProgLogStream;

        friend class ProgLog;
};


// general class
class ProgLog
{
public:
        explicit ProgLog(const char* FileName, bool isClear = true)
                : isFileCreated(true)
        {
                if (isClear) 
                {
                        ProgLogStream = new std::ofstream(FileName);
                        
                        if (ProgLogStream) 
                                (*ProgLogStream) << "***** This file was automatically created by ProgLog class *****" << std::flush;
                }
                else ProgLogStream = new std::ofstream(FileName, std::ofstream::app);                
                
                _progLog = new _ProgLog(ProgLogStream); 
        }

        explicit ProgLog(std::ostream& stream)
                : ProgLogStream(&stream), isFileCreated(false) 
        {
                _progLog = new _ProgLog(&stream); 
        }

        ~ProgLog()
        {
                if (_progLog) delete _progLog; 

                if (isFileCreated && ProgLogStream) delete ProgLogStream;
        }

// Adding info to log
// return: 0-OK, 1-cannot open log
        template <class T>
        int Add(const T& src)
        {
                if (isFileCreated && ProgLogStream) 
                        ProgLogStream->seekp(0, ProgLogStream->end);
                
                return Write(*ProgLogStream, src, true);
        }
        
        template <class T>  
        _ProgLog& operator << (const T& src)
        { 
                if (isFileCreated && ProgLogStream) 
                        ProgLogStream->seekp(0, ProgLogStream->end);
                
                Write(*ProgLogStream, src, true);
                
                return *_progLog;
        }

// Adding info to log-stream
//  isTime - "true" for adding time of writing (new line began)
// return: 0-OK, 1-cannot open log-stream
        template <class T>
        static int Write(std::ostream& stream, const T& src, bool isTime = true)
        {
                if (!stream || stream.fail()) return 1;
                
                if (isTime) 
                {
                        time_t ltime;
                        time(&ltime);

                        stream << std::endl << "-------------- " << ctime(&ltime);
                }

                stream << src << std::flush;
                
                return 0;
        }

// Adding info to log-file 
//  isTime - "true" for adding time of writing (new line began)
//  isClear - "true" for clearing log-file before writing
// return: 0-OK, 1-cannot open log-file
        template <class T>
        static int Write(const char* fileName, const T& src, bool isTime = true, bool isClear = false)
        {
                std::ofstream stream;

                if (isClear) stream.open(fileName);
                else stream.open(fileName, stream.app);
                
                if (!stream) return 1;
                
                return Write(stream, src, isTime);
        }

private:
        std::ostream* ProgLogStream;  // name of log-stream

        bool isFileCreated;

        _ProgLog* _progLog;  // auxiliary class for writing block of data to stream
};


#endif  
